/*******************************************************************************
*
*    File Name:   expaList.h
*
*    Description: 
*                 Include file for expaList.c
* 
*    
*
*******************************************************************************
*
*   History:
*
*	Rev.	Date		Name	               Description
*	-------------------------------------------------------------------------
*	1.0	4/20/04         Steve Bell             Created
*******************************************************************************/

#ifndef _EXPALIST_H
#define _EXPALIST_H

#define DYNAMIC

/* structure for rxns -- to be used as nodes in linked list */
struct rxnElt{
	char* rxnName; /*string*/
	struct rElt **headerPtr;
	struct rElt **currentPtr;
	struct rxnElt *Next;
};

/* structure for rxn elements -- to be used as nodes in linked list */
struct rElt{
	char* metName;/*string*/
	XTYPE coeff;	
	struct rElt *Next;
};

struct metElt{
	char* metName;/*string*/
	struct metElt *Next;
};



#ifdef _EXPALIST_C
/* initializes a list */
void metListInit(struct metElt** hPtr, struct metElt** cPtr);

/* returns 1 if list is empty, else 0 */
BOOLEAN metListIsEmpty(struct metElt** hPtr);

/* empties the list */
void metListMakeEmpty(struct metElt** hPtr, struct metElt** cPtr);

/* insert met at current position and advances
   current, returns 0 if error */
BOOLEAN metListInsert(char* newMet, struct metElt** cPtr);

/* removes met returns 1 if success, else 0 */
BOOLEAN metListRemove(char* oldMet, struct metElt** hPtr, struct metElt** cPtr);

/* sets current = header */
void metListZeroth(struct metElt** hPtr, struct metElt** cPtr);

/* set current to point to first element*/
/* returns 1 if success, else 0 */
BOOLEAN metListFirst(struct metElt** hPtr, struct metElt** cPtr);

/* finds findMet and sets current to point to it */
/* returns 1 if success, else 0 */
BOOLEAN metListFind(char* findMet, struct metElt** hPtr, struct metElt** cPtr);

/* advances current one step */
/* returns 1 if success, else 0 */
BOOLEAN metListAdvance(struct metElt** cPtr);

/***************************************rxn list****************************************************/

/* initializes a rxn list */
BOOLEAN rxnListInit(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);

/* insert newRxn at current position and advances
   current, returns 0 if error */
BOOLEAN rxnListInsert(char* newRxn, struct rxnElt** rxnCPtr);

/* finds findRxn and sets current to point to it */
/* returns 1 if success, else 0 */
BOOLEAN rxnListFind(char* findRxn, struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);

/* removes rxn returns 1 if success, else 0 */
BOOLEAN rxnListRemove(char* oldRxn, struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);

/* empties the list */
void rxnListMakeEmpty(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);

/* advances current one step */
/* returns 1 if success, else 0 */
BOOLEAN rxnListAdvance(struct rxnElt** rxnCPtr);

/* sets current = header */
void rxnListZeroth(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);

/***************************************r list****************************************************/
/* initializes a list */
void rListInit(struct rElt** hPtr, struct rElt** cPtr);

/* returns 1 if list is empty, else 0 */
BOOLEAN rListIsEmpty(struct rElt** hPtr);

/* empties the list */
void rListMakeEmpty(struct rElt** hPtr, struct rElt** cPtr);

/* insert r at current position and advances
   current, returns 0 if error */
BOOLEAN rListInsert(char* metName, XTYPE metCoeff, struct rElt** cPtr);

/* removes r returns 1 if success, else 0 */
BOOLEAN rListRemove(char* oldMet, struct rElt** hPtr, struct rElt** cPtr);

/* sets current = header */
void rListZeroth(struct rElt** hPtr, struct rElt** cPtr);

/* set current to point to first RowElt*/
/* returns 1 if success, else 0 */
BOOLEAN rListFirst(struct rElt** hPtr, struct rElt** cPtr);

/* finds findMet and sets current to point to it */
/* returns 1 if success, else 0 */
BOOLEAN rListFind(char* findMet, struct rElt** hPtr, struct rElt** cPtr);

/* advances current one step */
/* returns 1 if success, else 0 */
BOOLEAN rListAdvance(struct rElt** cPtr);

#else
extern void metListInit(struct metElt** hPtr, struct metElt** cPtr);
extern BOOLEAN metListIsEmpty(struct metElt** hPtr);
extern void metListMakeEmpty(struct metElt** hPtr, struct metElt** cPtr);
extern BOOLEAN metListInsert(char* newMet, struct metElt** cPtr);
extern BOOLEAN metListRemove(char* oldMet, struct metElt** hPtr, struct metElt** cPtr);
extern void metListZeroth(struct metElt** hPtr, struct metElt** cPtr);
extern BOOLEAN metListFirst(struct metElt** hPtr, struct metElt** cPtr);
extern BOOLEAN metListFind(char* findMet, struct metElt** hPtr, struct metElt** cPtr);
extern BOOLEAN metListAdvance(struct metElt** cPtr);

/***************************************rxn list****************************************************/

extern BOOLEAN rxnListInit(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);
extern BOOLEAN rxnListInsert(char* newRxn, struct rxnElt** rxnCPtr);
extern BOOLEAN rxnListFind(char* findRxn, struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);
extern BOOLEAN rxnListRemove(char* oldRxn, struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);
extern void rxnListMakeEmpty(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);
extern BOOLEAN rxnListAdvance(struct rxnElt** rxnCPtr);
extern void rxnListZeroth(struct rxnElt** rxnHPtr, struct rxnElt** rxnCPtr);

/***************************************r list****************************************************/
extern void rListInit(struct rElt** hPtr, struct rElt** cPtr);
extern BOOLEAN rListIsEmpty(struct rElt** hPtr);
extern void rListMakeEmpty(struct rElt** hPtr, struct rElt** cPtr);
extern BOOLEAN rListInsert(char* metName, XTYPE metCoeff, struct rElt** cPtr);
extern BOOLEAN rListRemove(char* oldMet, struct rElt** hPtr, struct rElt** cPtr);
extern void rListZeroth(struct rElt** hPtr, struct rElt** cPtr);
extern BOOLEAN rListFirst(struct rElt** hPtr, struct rElt** cPtr);
extern BOOLEAN rListFind(char* findMet, struct rElt** hPtr, struct rElt** cPtr);
extern BOOLEAN rListAdvance(struct rElt** cPtr);

#endif /* ifdef _list_c */

#endif /* end of file */


   


    
    

        
