/**************************************************************************
 * Most functions in this file are adapted from
 * Press, Teukolsky, Vetterling, and Flannerery, Numerical Recipes in C,
 * Second Edition, Cambridge University Press, 1992.
 * ************************************************************************/



#define _UTIL_C

#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include "expa.h"
#include "util.h"
#include "queue.h"

extern BOOLEAN verbose;

XTYPE *vector(UINT32 length)
{
	XTYPE* returnPtr;

	returnPtr = (XTYPE*) malloc( (unsigned) ( length * sizeof( XTYPE ) ) );
	if( returnPtr == NULL ){
		nrerror("allocation failure in vector()");
	}
	returnPtr--;
	return( returnPtr );


} /*vector()*/

UINT32 *uint32vector(UINT32 length)
{
	UINT32* returnPtr;

	returnPtr = (UINT32*) malloc( (unsigned) ( (length + NR_END) * sizeof( UINT32) ) );
	if( returnPtr == NULL ){
		nrerror("allocation failure in vector()");
	}
	return( returnPtr );


} /*vector()*/

BIT_TYPE *bitTypeVector(UINT32 length)
{
	BIT_TYPE* returnPtr;

	returnPtr = (BIT_TYPE*) malloc( (unsigned) ( length * sizeof( BIT_TYPE ) ) );
	if( returnPtr == NULL ){
		nrerror("allocation failure in vector()");
	}
	returnPtr--;
	return( returnPtr );


} /*vector()*/

XFLXTYPE *xflxvector(UINT32 length)
{
	XFLXTYPE* returnPtr;

	returnPtr = (XFLXTYPE*) malloc( (unsigned) ( (length + NR_END) * sizeof( XFLXTYPE ) ) );
	if( returnPtr == NULL ){
		nrerror("allocation failure in vector()");
	}
	return( returnPtr );


} /*vector()*/

XTYPE** xtypeMatrix(UINT32 nRows, UINT32 nCols)
{
	UINT32 row;
	XTYPE** returnPtr;

	/*allocate pointers to rows*/
	returnPtr = (XTYPE**) malloc( (unsigned) ( (nRows + NR_END) * sizeof( XTYPE* ) ) );
        if( returnPtr == NULL ){	
		nrerror("allocation failure 1 in xtypeMatrix()");
	}
	returnPtr += NR_END;
	returnPtr -= 1;

	/*allocate rows and set pointers to them*/
	for( row = 1; row <= nRows; row++ ){
		returnPtr[row] = (XTYPE*) malloc( (unsigned) nCols * sizeof( XTYPE ) );
		if( returnPtr[row] == NULL ){
			nrerror("allocation failure 2 in xtypeMatrix()");
		}
		(returnPtr[row])--;
	}

	/*return pointer to array of pointers to rows*/
	return( returnPtr );

} /*xtypeMatrix()*/


BIT_TYPE** bitTypeMatrix(UINT32 nRows, UINT32 nCols)
{
	UINT32 row;
	BIT_TYPE** returnPtr;

	/*allocate pointers to rows*/
	returnPtr = (BIT_TYPE**) malloc( (unsigned) ( (nRows + NR_END) * sizeof( BIT_TYPE* ) ) );
        if( returnPtr == NULL ){	
		nrerror("allocation failure 1 in matrix3()");
	}
	returnPtr += NR_END;
	returnPtr -= 1;

	/*allocate rows and set pointers to them*/
	for( row = 1; row <= nRows; row++ ){
		returnPtr[row] = (BIT_TYPE*) malloc( (unsigned) nCols * sizeof( BIT_TYPE ) );
		if( returnPtr[row] == NULL ){
			nrerror("allocation failure 2 in matrix3()");
		}
		(returnPtr[row])--;
	}

	/*return pointer to array of pointers to rows*/
	return( returnPtr );

} /*uint32Matrix()*/


void freeVector(XTYPE* vector)
{
	vector++;
	free( (FREE_ARG) vector );
} /*freeVector()*/

void freexflxVector(XFLXTYPE* vector)
{
	free( (FREE_ARG) (vector + 1 - NR_END) );
} /*freexflxVector()*/

void freeuint32Vector(UINT32* vector)
{
	free( (FREE_ARG) (vector + 1 - NR_END) );
} /*freeuint32Vector()*/

void freebitTypeVector(BIT_TYPE* vector)
{
	vector++;
	free( (FREE_ARG) (vector + 1 - NR_END) );
} /*freeuint32Vector2()*/


void freextypeMatrix(XTYPE** mat, UINT32 nRows)
{
	UINT32 row;
	for( row = 1; row <= nRows; row++ ){
		free( (FREE_ARG) (mat[row] + 1) );
	}
	free( (FREE_ARG) mat );
}/*freeXtypeMatrix()*/

void freebitTypeMatrix(BIT_TYPE** mat, UINT32 nRows)
{
	UINT32 row;
	for( row = 1; row <= nRows; row++ ){
		free( (FREE_ARG) (mat[row] + 1) );
	}
	free( (FREE_ARG) mat );
}/*freebitTypeMatrix()*/


void nrerror(char error_text[])
{
	fprintf(stderr, "run-time error...\n");
	fprintf(stderr, "%s\n", error_text);
	fprintf(stderr, "...now exiting to system...\n");
	exit(4);
} /*nrerror()*/

void writeMat( XTYPE **mat, BIT_TYPE **bitArray, UINT32 rowSize)
{
	UINT32 numberOfXtypes = NCOLS/XTYPESIZE;
	UINT32 numberOfResiduals = NCOLS%XTYPESIZE;
	UINT32 i,j,k,l; 
	XTYPE outXtype;
	BIT_TYPE testBit, xBit;
	FILE *fp;

	if( ( fp = fopen("Paths.txt", "w") ) == NULL ){
		fprintf(stderr, "could not open file Paths.txt");
		exit(2);
	}
	
	
	for( i = 1; i <= rowSize; i++ ){
		l = 1;
		if( bitArray[i] == NULL ){
			continue;
		}
		for( j = 1; j <= numberOfXtypes; j++ ){
			testBit = HIGH_BIT;
			xBit = bitArray[i][j];
			for( k = 1; k <= XTYPESIZE; k++ ){
				if( (xBit & testBit) != 0 ){
					outXtype = mat[i][l];
					l++;
				}
				else{
					outXtype = 0;
				}

				if( outXtype >= 0 ){
					fprintf(fp, "   %ld", outXtype);
				}
				else{
					fprintf(fp, "  %ld", outXtype);
				}
				testBit = testBit >> 1;
			}
		}
		testBit = HIGH_BIT;
		xBit = bitArray[i][j];
		for( k = 1; k <= numberOfResiduals; k++ ){
			if( (xBit & testBit) != 0 ){
				outXtype = mat[i][l];
				l++;
			}
			else{
				outXtype = 0;
			}

			if( outXtype >= 0 ){
				fprintf(fp, "   %ld", outXtype);
			}
			else{
				fprintf(fp, "  %ld", outXtype);
			}
			testBit = testBit >> 1;
		} /*for(j)*/
		fprintf(fp, "\n");
	} /*for(i)*/
	if( rowSize == 0 ){
		for( k = 1; k <= NCOLS; k++ ){
			fprintf(fp, "%ld  ", 0);
		}
	}
	fclose(fp);
		
} /*writeMat()*/


void writeMat2( XTYPE **mat, UINT32 **bitArray, UINT32 rowSize)
{
	FILE* matFile;
	FILE* colFile;
	UINT32 i,j,k,l, subtractRows, col, nrows, ncols, nmets; 
	XTYPE outXtype;
	UINT32 testBit, xBit;
	UINT32 numberOfColumns = 0;

	/*file for pathways*/
	if( ( matFile = fopen("mat.txt", "w") ) == NULL ){
		fprintf(stderr, "Cannot open path output file\n");
		exit(2);
	}

	/*file for unprocessed columns*/
	if( ( colFile = fopen("cols.txt", "w") ) == NULL ){
		fprintf(stderr, "Cannot open path output file\n");
		exit(2);
	}
	
	subtractRows = 0;
	for( i = 1; i <= rowSize; i++ ){
		l = 1;
		if( bitArray[i] == NULL ){
			subtractRows++;
			continue;
		}
		for( j = 1; j <= NUMBER_OF_XTYPES; j++ ){
			testBit = HIGH_BIT;
			xBit = bitArray[i][j];
			for( k = 1; k <= XTYPESIZE; k++ ){
				if( (xBit & testBit) != 0 ){
					outXtype = mat[i][l];
					l++;
				}
				else{
					outXtype = 0;
				}

				if( outXtype >= 0 ){
					fprintf(matFile, "   %ld", outXtype);
				}
				else{
					fprintf(matFile, "  %ld", outXtype);
				}
				testBit = testBit >> 1;
			}
		}/*for(j)*/
		testBit = HIGH_BIT;
		xBit = bitArray[i][j];
		for( k = 1; k <= NUMBER_OF_RESIDS; k++ ){
			if( (xBit & testBit) != 0 ){
				outXtype = mat[i][l];
				l++;
			}
			else{
				outXtype = 0;
			}

			if( outXtype >= 0 ){
				fprintf(matFile, "   %ld", outXtype);
			}
			else{
				fprintf(matFile, "  %ld", outXtype);
			}
			testBit = testBit >> 1;
		} /*for(k)*/
		fprintf(matFile, "\n");
	} /*for(i)*/	
	if( rowSize == 0 ){
		for( k = 1; k <= (NCOLS + NROWS); k++ ){
			fprintf(matFile,"%ld  ", 0);
		}
		fprintf(matFile,"\n");
	}
	fclose(matFile);
	nrows = rowSize - subtractRows;
	ncols = NUMBER_OF_XTYPES * XTYPESIZE + NUMBER_OF_RESIDS;
	nmets = ncols - NCOLS;
	if( verbose == my_true ){
		printf("printing path matrix ...\n");
		printf("file is mat.txt\n");
		printf("dimension of matrix: %ld X %ld\n", nrows, ncols);
		printf("number of metabolites: %ld \n", nmets);
		printf("printing remaining columns ...\n");
		printf("file is cols.txt\n");
	}
	while( GetNumberOfWords() > 0 ){
		GetWord( (UINT32*) &col );
		fprintf(colFile, "%ld  ", (col - NCOLS));
		numberOfColumns++;
	}
	fprintf(colFile, "\n");
	if( verbose == my_true ){
		printf("number of columns: %ld\n", numberOfColumns);
	}
	fclose(colFile);
		
} /*writeMat2()*/


void writeStoickMat( XTYPE** sMat, UINT32 nrows, UINT32 ncols )
{
	UINT32 row, col;
	XTYPE outXtype;
	for( row = 1; row <= NROWS; row++ ){
		for( col = 1; col <= NCOLS; col++ ){
			outXtype = sMat[row][col];
			if( outXtype >= 0 ){
				printf("    %ld", outXtype);
			}
			else{
				printf("   %ld", outXtype);
			}
		}
		printf("\n");
	}
	printf("\n");
} /*writeStoickMat()*/

